package be.iminds.ilabt.jfed.experiment;

/**
 * User: twalcari
 * Date: 1/30/2015
 * Time: 11:05
 */
public enum ExperimentState {

    /**
     * Initial status of a new Experiment.
     * <p/>
     * The slice is to be created at the slice authority (It is currently unallocated).
     * <p/>
     * Next step in lifecycle will be {@link #CREATED}
     */
    PENDING,

    /**
     * Initial status of an existing slice, of which we know nothing except for it's URN.
     * <p/>
     * While in this state, basic information about the slice is reconstructed by contacting the slice authority.
     * After that, all necessary sources are contacted to reconstruct the full state of all the slivers.
     * <p/>
     * This state transfers to {@link #RESTORED} or {@link #RENEW_EXISTING}
     */
    RESTORING,

    /**
     * State of an existing slice that will be used for a new experiment, when all its state is known.
     * <p/>
     * If needed, the slice will be renewed in this state.
     * <p/>
     * This state transfers to the state {@link #CREATED} (or {@link #FAILING} if the slice cannot be renewed)
     */
    RENEW_EXISTING,

    /**
     * This state is reached when the status of the restored experiment is fully known.
     *<p/>
     * This state transfers to status {@link #WAIT_FOR_READY},
     * {@link #FAILING}, {@link #EXPIRED},
     * {@link #TESTING_CONNECTIVITY} or {@link #FUTURE_RESERVATION} as appropriate.
     */
    RESTORED,

    /**
     * The slice is allocated, resource must be requested, but no resources have been requested yet (requested = allocated and provisioned).
     * <p/>
     * If the resources are only available in the future, this state transfers to {@link #FUTURE_RESERVATION}
     * <p/>
     * Otherwise, this state transfers to {@link #ALLOCATING}
     */
    CREATED,


    /**
     * The resources mentioned in experiment.getNewRequestRespecSource() need to be allocated.
     * Only when the allocation was succesful, we can formally provision them.
     *<p/>
     * When the allocation request was succesful, we change the state to {@link #PROVISIONING} if the resources are
     * available immediately, or to {@link #FUTURE_RESERVATION} if we need to wait for the reservation to begin.
     */
    ALLOCATING,

    /**
     * The resources of the slice are not yet provisionable, we have to wait until the correct time arrives.
     * <p/>
     * Once the reservation start time passes, the slice can transfer to {@link #PROVISIONING}.
     * <p/>
     * The slice contains a manifest of the resources it has been promised.
     */
    FUTURE_RESERVATION,

    /**
     * The resources of the slice are currently being provisioned.<br/>
     * During this state, the slice receives the manifests of all slivers.
     * This state includes waiting until all resources are ready (or there is an error),
     * by periodically calling Status.
     * <p/>
     * TODO: This state needs to be split (see {@link #WAIT_FOR_READY} and {@link #SETUP_SOFTWARE})
     * <p/>
     * When provisioning is done, the resources must be tested (transfer to status {@link #WAIT_FOR_READY} or {@link #FAILING}).
     */
    PROVISIONING,

    /**
     * Wait for the resources to become ready.
     *<p/>
     * This step occurs:<br/>
     *   - after a restore, if the resources are not yet ready<br/>
     *   - always when starting an experiment.
     * <p/>
     * This state transfers to {@link #TESTING_CONNECTIVITY} or {@link #FAILING}
     */
    WAIT_FOR_READY,

    /**
     * Connectivity to all resources is tested (tcp connection setup to the ssh port).
     * <p/>
     * This state transfers to status {@link #TESTING_LINKS}
     */
    TESTING_CONNECTIVITY,

    /**
     * The links between resources are tested.
     *   (This will do nothing if there are no links, or the user did not request a link test.)
     * <p/>
     * This state transfers to status {@link #SETUP_SOFTWARE} or {@link #FAILING}
     */
    TESTING_LINKS,

    /**
     * Run ansible scripts and ESpec functionality.
     *<p/>
     * This state transfers to status {@link #READY} or {@link #FAILING}
     */
    SETUP_SOFTWARE,


    /**
     * The slice is fully provisioned and tested.
     * <p/>
     * The slice remains in this state, until the expiration time of one or more resources is reached.
     * After that, it moves to the state {@link #EXPIRING}, in which this is checked.
     */
    READY,

    /**
     * User requested to stop all resources in the experiment.
     */
    DELETING,

    /**
     * One or more slivers and/or slice is expiring. We need to check it to make sure it really happened!
     */
    EXPIRING,
    /**
     * All resources of the slice have expired
     */
    EXPIRED,

    /**
     * There are no more active resources in the experiment
     */
    EMPTY,

    /**
     * While waiting for the experiment to get ready, a timeout occured.
     *
     * the state transfers to {@link #UNKNOWN}, {@link #DELETING} or returns to {@link #WAIT_FOR_READY}
     */
    TIMEOUT_WAITING,

    /**
     * If a fatal error occurs, and the user should be asked if he wants to delete the resources, the ExperimentState transfers to failing.
     * After asking the user, and deleting the resources (if the user agreed to it), the state transfers to {@link #FAILED}
     */
    FAILING,

    /**
     * If there occurs a fatal error, the slice transfers to {@link #FAILED}. The user cannot use this slice anymore.
     */
    FAILED,

    /**
     * The experiment is in an unknown state. Nothing will be done/checked,
     * until the user tries to manually check/change state.
     * One way this state is reached, is when the user chooses "stop waiting" at TIMEOUT_WAITING
     */
    UNKNOWN;

    public boolean isTerminated() {
        return this == DELETING || this == EMPTY || this == EXPIRED;
    }

    public boolean isActiveState(){
        return this != FAILING && this != FAILED && this != EMPTY && this != EXPIRED;
    }
}
